class BuildInfo:  # pylint: disable=too-few-public-methods
    python_version = '3.10.15'
    os_name = 'posix:ubuntu'
    version = '0.0.4'
    git_sha = 'da562b7c91033f5d29f4c3e04479eec01e215377'
    git_branch = 'master'
    git_uncommitted = [
    ]
    git_unpushed = [
    ]
